package frc.robot.subsystems;

import edu.wpi.first.units.Distance;
import edu.wpi.first.units.Measure;
import edu.wpi.first.units.MutableMeasure;
import edu.wpi.first.units.Units;
import edu.wpi.first.units.Velocity;
import edu.wpi.first.units.Voltage;
import edu.wpi.first.wpilibj.RobotController;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.sysid.SysIdRoutine;

public class ElevatorSysID extends Elevator {
    private MutableMeasure<Voltage> appliedVoltage;
    private MutableMeasure<Distance> distance;
    private MutableMeasure<Velocity<Distance>> velocity;

    private SysIdRoutine sysIDRoutine;

    public ElevatorSysID() {
        super();

        appliedVoltage = MutableMeasure.mutable(Units.Volts.of(0));
        distance = MutableMeasure.mutable(Units.Meters.of(0));
        velocity = MutableMeasure.mutable(Units.MetersPerSecond.of(0));

        sysIDRoutine = new SysIdRoutine(
            new SysIdRoutine.Config(), 
            new SysIdRoutine.Mechanism(
                (Measure<Voltage> volts) -> {
                    motor1.setVoltage(volts.in(Units.Volts));
                }, 
                (log) -> {
                    log.motor("elevator")
                        .voltage(
                            appliedVoltage.mut_replace(
                                motor1.get() * RobotController.getBatteryVoltage(),
                                Units.Volts
                            )
                        )
                        .linearPosition(
                            distance.mut_replace(encoder.getDistance(), Units.Meters)
                        )
                        .linearVelocity(
                            velocity.mut_replace(encoder.getRate(), Units.MetersPerSecond)
                        );
                }, 
                this
            )
        );
    }

    public Command sysIdQuasistatic(SysIdRoutine.Direction direction) {
        return sysIDRoutine.quasistatic(direction);
    }

    public Command sysIdDynamic(SysIdRoutine.Direction direction) {
        return sysIDRoutine.dynamic(direction);
    }
}
